﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication1
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Car[] cars = new Car[20];

        public MainWindow()
        {
            InitializeComponent();

            for (int i = 0; i < cars.Length; i++)
            {
                cars[i] = new Car();
            }

            timer.Interval = TimeSpan.FromMilliseconds(16.0);
            timer.Tick += animate;
            timer.Start();
        }

        System.Windows.Threading.DispatcherTimer timer
            = new System.Windows.Threading.DispatcherTimer();

        double[] averages = new double[1000];

        int frameNumber;

        DateTime lastTime = DateTime.Now; // nach Videoaufnahme hinzugefügt

        void animate(object sender, EventArgs e)
        {
            DateTime now = DateTime.Now; // nach Videoaufnahme hinzugefügt
            TimeSpan interval = now - lastTime; // nach Videoaufnahme hinzugefügt
            for (int i = 0; i < cars.Length; i++)
            {
                // den dichtliegendsten Vorgänger finden
                double minDist = 42.0;
                int minDistIndex = -1;
                for (int j = 0; j < cars.Length; j++)
                {
                    if (j == i)
                    {
                        continue;
                    }
                    double distance = cars[j].Position - cars[i].Position;
                    distance -= Math.Floor(distance);
                    if (distance < minDist)
                    {
                        minDist = distance;
                        minDistIndex = j;
                    }
                }

                if (minDist < 0.02 && cars[i].Speed > cars[minDistIndex].Speed)
                {
                    // Geschwindigkeit reduzieren
                    cars[i].Speed = cars[minDistIndex].Speed;
                }

                // nach Videoaufnahme geändert:
                // intervall gemessen, statt vom Timer geholt,
                // weil der Timer das Intervall nicht genau einhält
                cars[i].Move(interval);
            }
            lastTime = now; // nach Videoaufnahme hinzugefügt

            double minSpeed = double.MaxValue;
            double maxSpeed = 0.0;
            double sumSpeed = 0.0;
            for (int i = 0; i < cars.Length; i++)
            {
                double s = cars[i].Speed;
                minSpeed = Math.Min(minSpeed, s);
                maxSpeed = Math.Max(maxSpeed, s);
                sumSpeed += s;
            }
            labelMin.Content = minSpeed;
            labelMax.Content = maxSpeed;
            double avg = sumSpeed / cars.Length;
            labelMean.Content = avg;

            if (frameNumber < averages.Length) // nach Videoaufnahme hinzugefügt
            {
                Title = frameNumber.ToString();
                averages[frameNumber] = avg;
                frameNumber++;
            }

            myCanvas.Children.RemoveRange(3, cars.Length);
            for (int i = 0; i < cars.Length; i++)
            {
                cars[i].Draw(myCanvas);
            }
        }
    }

    class Car
    {
        double position; // Zahl der Runden
        public double Position
        {
            get { return position; }
        }

        double speed; // Geschwindigkeit in Runden pro Minute
        public double Speed
        {
            get { return speed; }
            set { speed = value; }
        }

        static Random random = new Random();

        public Car()
        {
            position = random.NextDouble();
            speed = 8.0 + 2.0 * random.NextDouble();
        }

        public void Draw(Canvas canvas)
        {
            Ellipse e = new Ellipse();
            e.Width = 5.0;
            e.Height = 5.0;
            e.Fill = Brushes.Blue;
            double angle = 2.0 * Math.PI * position;
            Canvas.SetLeft(e, 160.0 + 150.0 * Math.Sin(angle));
            Canvas.SetTop(e, 160.0 - 150.0 * Math.Cos(angle));
            canvas.Children.Add(e);
        }

        public void Move(TimeSpan timeSpan)
        {
            position += speed * timeSpan.TotalMinutes;
        }
    }
}
